/***************************************************************************
 *   Weather module for Kadu communicator                                  *
 *   2004 Micha Maek (bysk)                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <stdlib.h>
#include <stdio.h>

#include <qprocess.h>
#include <qcombobox.h>
#include <qmessagebox.h>
#include <qvgroupbox.h>
#include <qcheckbox.h>
#include <qspinbox.h>
#include <qclipboard.h>

#include "kadu.h"
#include "userbox.h"
#include "config_dialog.h"
#include "message_box.h"

#ifdef EXTERNAL
#include "modules/hint_manager.h"
#else
#include "../hints/hint_manager.h"
#endif

#include "weather.h"

Weather* weather_session;

extern "C" int weather_init()
{
    weather_session = new Weather();

    return 0;
}

extern "C" void weather_close()
{	
    delete weather_session;
}


/**
	WeatherCodeDialog
**/

int WeatherCodeDialog::config_pos = 0;

WeatherCodeDialog::WeatherCodeDialog(QListViewItem* item, ConfigFile* cf, const QStringList& wCfFiles,
		const QStringList& wCfNames, int actual, QStringList& lastLoc, QWidget* parent)
:
	QDialog(parent, "WeatherCodeDialog"),
	idSearch(cf, wCfFiles),
	listItem(item),
	moduleConfig(cf),
	weatherConfigNames(wCfNames),
	weatherConfigFiles(wCfFiles),
	lastLocList(lastLoc)
{
	if(actual != -1)
		config_pos = actual;

	setWFlags(WDestructiveClose/* | WShowModal*/);

	QGridLayout *grid= new QGridLayout(this, 1,0, 5, 10);
	
	QHBox* config_box = new QHBox(this);
	new QLabel(tr("Weather server: "), config_box);
	c_config = new QComboBox(false,config_box);
	c_config->insertStringList(weatherConfigNames);
	c_config->setCurrentItem(config_pos);
	
	QVGroupBox* search_group_box = new QVGroupBox(tr("Search location code"),this);
	
	QHBox* search_box = new QHBox(search_group_box);
	new QLabel(QString(tr("Name: ")), search_box);
	c_name = new QComboBox(search_box);
	c_name->setEditable(true);
	c_name->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed));
	b_search = new QPushButton(search_box);
	b_search->setText(tr("Search"));
	b_search->setMaximumWidth(200);
	
	lv_search = new QListView(search_group_box);
	lv_search->setSorting(-1);
	lv_search->addColumn(tr("Location"));
	lv_search->addColumn(tr("Code"));
	lv_search->setAllColumnsShowFocus(true);
	lv_search->setColumnWidthMode(0, QListView::Maximum);
	lv_search->setColumnWidthMode(1, QListView::Maximum);
	lv_search->setResizeMode(QListView::LastColumn);
	lv_search->setSorting(0);
	
	p_linking = new QProgressBar(0, search_group_box);
	p_linking->hide();
	
	QHBox* bottom = new QHBox(this);
	new QLabel(QString(tr("Location code for %1: ")).arg(item->text(0)),bottom);
	e_id = new QLineEdit(item->text(1), bottom);
	
	QHBox* okcancel=new QHBox(this);
	
	b_ok = new QPushButton(okcancel);
	b_ok->setText(tr("OK"));
	b_ok->setMaximumWidth(200);
	b_ok->setSizePolicy(QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed));
	b_ok->setIconSet(icons_manager.loadIcon("OkWindowButton"));
	
	b_cancel = new QPushButton(okcancel);
	b_cancel->setText(tr("Cancel"));
	b_cancel->setMaximumWidth(200);
	b_cancel->setSizePolicy(QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed));
	b_cancel->setIconSet(icons_manager.loadIcon("CancelWindowButton"));
	
	grid->addWidget(config_box, 0, 0);
	grid->addWidget(search_group_box, 1, 0);
	grid->addWidget(bottom, 2, 0);
	grid->addWidget(okcancel, 3, 0, Qt::AlignRight);
	
	timer = new QTimer(this);
	
	setCaption(tr("Edit location code"));
	
	for(int i=0; i<10; i++)
	{
		if(!lastLocList[i].isEmpty())
			c_name->insertItem(lastLocList[i], i);
	}
	c_name->setFocus();
	c_name->setCurrentText("");
	
	connect(b_search, SIGNAL(clicked()), this, SLOT(searchButton()));
	connect(b_ok, SIGNAL(clicked()), this, SLOT(okButton()));
	connect(b_cancel, SIGNAL(clicked()), this, SLOT(close()));
	connect(lv_search, SIGNAL(selectionChanged()), this, SLOT(searchSelChanged()));
	connect(timer, SIGNAL(timeout()), this, SLOT(downloadingProgress()));
	
	connect(&idSearch, SIGNAL(finished()), this, SLOT(downloadingFinished()));
	connect(&idSearch, SIGNAL(error()), this, SLOT(downloadingError()));
	//connect(&idSearch, SIGNAL(redirected(QString)), this, SLOT(downloadingRedirected(QString)));
	connect(c_name, SIGNAL(activated(int)), this, SLOT(searchButton()));
}

WeatherCodeDialog::~WeatherCodeDialog()
{	
	disconnect(c_name, SIGNAL(activated(int)), this, SLOT(searchButton()));
	disconnect(b_search, SIGNAL(clicked()), this, SLOT(searchButton()));
	disconnect(b_ok, SIGNAL(clicked()), this, SLOT(okButton()));
	disconnect(b_cancel, SIGNAL(clicked()), this, SLOT(close()));
	disconnect(lv_search, SIGNAL(selectionChanged()), this, SLOT(searchSelChanged()));
	disconnect(timer, SIGNAL(timeout()), this, SLOT(downloadingProgress()));
	
	disconnect(&idSearch, SIGNAL(finished()), this, SLOT(downloadingFinished()));
	disconnect(&idSearch, SIGNAL(error()), this, SLOT(downloadingError()));
	//disconnect(&idSearch, SIGNAL(redirected(QString)), this, SLOT(downloadingRedirected(QString)));
}

void WeatherCodeDialog::okButton()
{
	QString code = e_id->text();
	listItem->setText(1, code);
	
	if(code.isEmpty())
	{
		listItem->setText(2, "");
		listItem->setText(3, "");
	}
	else
	{
		listItem->setText(2, weatherConfigNames[c_config->currentItem()]);
		listItem->setText(3, weatherConfigFiles[c_config->currentItem()]);
	}
	close();
}

void WeatherCodeDialog::searchButton()
{
	lv_search->clear();
	
	config_pos = c_config->currentItem();
	QString loc_name = c_name->currentText();
	if(!idSearch.findID(loc_name, config_pos))
		MessageBox::msg(tr("List of locations not available for this server. Type part of location name."));
	else
	{
		p_linking->reset();
		p_linking->show();
		timer->start(100,false);
	}
	
}

void WeatherCodeDialog::downloadingFinished()
{
	timer->stop();
	p_linking->hide();
	
	const QStringList& names = idSearch.getNames();
	const QStringList& codes = idSearch.getCodes();

	uint count = names.count();
	for(uint i = 0; i < count; i++)
	{
		new QListViewItem(lv_search,names[i], codes[i]);
	}
	
	if(count == 0)
		MessageBox::msg(tr("Search location not found"));
	else if(count == 1)
	{
		QString location = c_name->currentText();
		if(lastLocList.find(location) == lastLocList.end())
		{
			lastLocList.push_front(location);
			c_name->insertItem(location, 0);
			if(lastLocList.count() > 10)
				lastLocList.remove(lastLocList.at(10));
		}
	
		lv_search->setCurrentItem(lv_search->firstChild());
		lv_search->setSelected(lv_search->firstChild(), true);
	}
}

void WeatherCodeDialog::downloadingError()
{
	p_linking->hide();
	timer->stop();
	MessageBox::wrn(tr("Downloading failed"));
}

void WeatherCodeDialog::downloadingRedirected(QString link)
{
	p_linking->hide();
	timer->stop();
	
	const QStringList& names = idSearch.getNames();
	const QStringList& codes = idSearch.getCodes();
	
	if(!(*codes.begin()).isEmpty())
	{
		QListViewItem* item = new QListViewItem(lv_search,*names.begin(), *codes.begin());
		lv_search->setCurrentItem(item);
		lv_search->setSelected(item, true);
	}
}

void WeatherCodeDialog::downloadingProgress()
{
	p_linking->setProgress(p_linking->progress()+10);
}

void WeatherCodeDialog::searchSelChanged()
{
	e_id->setText(lv_search->currentItem()->text(1));
}

/**
	ShowForecastDialog
**/

ShowForecastDialog::ShowForecastDialog(ForecastContener& fl, const QString& configFile,
	const QString& locID, ConfigFile* cf, QWidget* parent)
:
	QDialog(parent, "ShowForecastDialog"),
	downloader(cf,forecast),
	moduleConfig(cf),
	savedForecast(fl),
	currentPage(-1)
{
	setWFlags(WDestructiveClose);
	setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
	//setMinimumHeight(50);
	//updateGeometry();
	
	vlayout = new QVBoxLayout(this);
	vlayout->setAutoAdd(true);
	vlayout->setMargin(5);
	
	e_message = new QLabel(this);
	e_message->setText(tr("Loading data... Please wait..."));
	e_message->setAlignment(Qt::AlignTop);
	e_message->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
	//e_message->setTextFormat(Qt::PlainText);
	
	btn_box = new QHButtonGroup(this);
	btn_box->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Minimum);
	btn_box->setExclusive(true);
	btn_box->setFrameShape(QFrame::NoFrame);
	btn_box->setInsideSpacing(1);
	btn_box->setInsideMargin(0);
	
	updateGeometry();
	
	timer = new QTimer(this);
	
	//resize(300,300);
	
	contextMenu = new QPopupMenu(this);
	contextMenu->insertItem(tr("Copy"), this, SLOT(menuCopy()));
	contextMenu->insertItem(tr("Go to Web page"), this, SLOT(menuGoToPage()));
	
	/*
	for(UserList::const_iterator u = userlist.begin(); u != userlist.end(); u++)
	{
		if(QString::number((*u).uin()) == uin)
		{
			setCaption((*u).altNick() + " - " + tr("Forecast"));
			break;
		}
	}
	if(caption().isEmpty())
		setCaption(tr("Forecast"));
	*/
	
	connect(btn_box, SIGNAL(clicked(int)), this, SLOT(dayClicked(int)));
	connect(timer, SIGNAL(timeout()), this, SLOT(downloadingProgress()));
	connect(&downloader, SIGNAL(finished()), this, SLOT(downloadingFinished()));
	connect(&downloader, SIGNAL(error()), this, SLOT(downloadingError()));
	
	if(savedForecast.getForecast(configFile, locID, forecast, moduleConfig->readNumEntry("Weather","KeepFor")))
		showForecast();
	else
	{
		p_linking = new QProgressBar(0, this);
		
		p_linking->reset();
		timer->start(100,false);
		downloader.downloadForecast(configFile, locID);
	}
}

ShowForecastDialog::~ShowForecastDialog()
{
	disconnect(&downloader, SIGNAL(error()), this, SLOT(downloadingError()));
	disconnect(&downloader, SIGNAL(finished()), this, SLOT(downloadingFinished()));
	disconnect(timer, SIGNAL(timeout()), this, SLOT(downloadingProgress()));
	disconnect(btn_box, SIGNAL(clicked(int)), this, SLOT(dayClicked(int)));
}

void ShowForecastDialog::downloadingFinished()
{	
	p_linking->hide();
	timer->stop();
	
	if(moduleConfig->readNumEntry("Weather","KeepFor") > 0)
		savedForecast.add(forecast);
	
	showForecast();
}

void ShowForecastDialog::downloadingError()
{
	p_linking->hide();
	timer->stop();
	MessageBox::wrn(tr("Downloading failed"));
	close();
}

void ShowForecastDialog::downloadingProgress()
{
	p_linking->setProgress(p_linking->progress()+10);
}

void ShowForecastDialog::showForecast()
{
	QValueList<ForecastDay>::iterator it;
	for(it = forecast.Days.begin(); it != forecast.Days.end(); it++)
	{
		ForecastDay& day = (*it);
		QPushButton* btn = new QPushButton(btn_box);
		btn->setToggleButton(true);
		btn->setPixmap(QPixmap(day["Icon"]));
		btn->show();
		btn_box->insert(btn);
		
		QToolTip::add(btn, "<b>"+day["Name"] + "</b><br>" + day["Temperature"]);
	}
	btn_box->setButton(0);
	dayClicked(0);
	
	//updateGeometry();
	//setMinimumSize(vlayout->sizeHint());
	//setFixedSize(vlayout->minimumSize());
	
	resize(350,200);
	
	//resize(vlayout->sizeHint());
	//setMinimumSize(vlayout->sizeHint());
	//setMaximumHeight(vlayout->sizeHint().height());
}

void ShowForecastDialog::dayClicked(int id)
{
	currentPage = id;

	ForecastDay& day = forecast.Days[id];
	
	QString Data("");
	Data += "<b><u>" + forecast.LocationName + "</u> " + day["Name"] +
			"</b><hr><table width=\"100%\"><tr><td align=\"left\">";
	Data += "<img src=\""+day["Icon"]+"\"><br>";
	Data += "<b>"+day["Temperature"]+"</b>";
	Data += "</td><td>";
	
	bool first = true;
	for(ForecastDay::iterator it = day.begin(); it != day.end(); it++)
	{
		if(it.key() != "Name" && it.key() != "Icon" && it.key() != "Temperature")
		{
			if(first)
				first = false;
			else
				Data += "<br>";
			Data += tr(it.key()) + ": " + it.data();
		}
	}
	
	Data += "</td></tr></table>";
	Data += tr("Forecast received from ")+forecast.serverName;
	
	e_message->setText(Data);
}

void ShowForecastDialog::mousePressEvent(QMouseEvent * e)
{
	if(e->button() != Qt::RightButton || currentPage < 0)
		return;
	
	contextMenu->popup(mapToGlobal(e->pos()));
}

void ShowForecastDialog::menuCopy()
{
	ForecastDay& day = forecast.Days[currentPage];
	QString Data = forecast.LocationName  + " - " + day["Name"] + "\n";
	
	bool first = true;
	for(ForecastDay::iterator it = day.begin(); it != day.end(); it++)
	{
		if(it.key() != "Name" && it.key() != "Icon")
		{
			if(first)
				first = false;
			else
				Data += "\n";
			Data += tr(it.key()) + ": " + it.data();
		}
	}

	QApplication::clipboard()->setText(Data);
}

void ShowForecastDialog::menuGoToPage()
{
	QProcess browser;
	QStringList args;
	
	args = QStringList::split(" ", config_file.readEntry("Chat","WebBrowser"));
	
	ConfigFile wConfig(dataPath("kadu/modules/data/weather/" + forecast.config));
	QString url = wConfig.readEntry("Default","Default host") + "/" + wConfig.readEntry("Default","Default path");
	url.replace("%s", forecast.LocationID);
	args.push_back(url);
	
	browser.setArguments(args);
	browser.start();
}

/**
	Weather
**/
Weather::Weather()
{
	QString config_path = ggPath("weather.conf");
	printf("%s\n", config_path.ascii());
	moduleConfig = new ConfigFile(config_path);
	
	QDir dir(dataPath("kadu/modules/data/weather"),"*.ini");
	dir.setFilter(QDir::Files);
	for(unsigned int i=0; i<dir.count(); i++)
	{
		QString name=dir[i];
		weatherConfigFiles.append(name.left(name.length()));
		
		ConfigFile wConfig(dataPath("kadu/modules/data/weather/"+name));
		weatherConfigNames.append(wConfig.readEntry("Header","Name"));
	}
	
	for(int i=0; i<10; i++)
		lastLocList.push_back(moduleConfig->readEntry("LastLoc", QString("Location%1").arg(i+1)));
	
	downloader = new GetForecast(moduleConfig, forecast);
	
	moduleConfig->addVariable("Weather", "HintFont", QFont("Helvetica", 11));
	moduleConfig->addVariable("Weather", "HintDay", 0);
	moduleConfig->addVariable("Weather", "DescriptionDay", 0);
	moduleConfig->addVariable("Weather", "HintText", tr("<u>%l</u> - <b>%d:</b><br>Temperature: %t<br>Pressure: %p"));
	moduleConfig->addVariable("Weather", "DescriptionText", tr("Temperature in %l: %t"));
	
	ConfigDialog::addTab(QT_TRANSLATE_NOOP("@default", "Weather"),"WeatherTab");
	
	ConfigDialog::addVBox("Weather", "Weather", "box");
	ConfigDialog::addVGroupBox("Weather", "box", QT_TRANSLATE_NOOP("@default", "Weather for contacts"));
	ConfigDialog::addListView("Weather", "Weather for contacts", "loc_list");
	ConfigDialog::addHBox("Weather", "Weather for contacts", "button_box");
	ConfigDialog::addPushButton("Weather", "button_box", QT_TRANSLATE_NOOP("@default", "Find"),"LookupUserInfo");
	ConfigDialog::addPushButton("Weather", "button_box", QT_TRANSLATE_NOOP("@default", "Edit"),"ChangeSelectPathDialogButton");
	ConfigDialog::addPushButton("Weather", "button_box", QT_TRANSLATE_NOOP("@default", "Clear"),"RemoveSelectPathDialogButton");
	ConfigDialog::addPushButton("Weather", "button_box", QT_TRANSLATE_NOOP("@default", "Find all..."),"LookupUserInfo");
	ConfigDialog::addVGroupBox("Weather", "box", QT_TRANSLATE_NOOP("@default", "Options"));
	ConfigDialog::addCheckBox(moduleConfig, "Weather", "Options",
			QT_TRANSLATE_NOOP("@default","Show \"Forecast for...\" in main menu"),
			"ForecastFor", true);
	ConfigDialog::addSpinBox(moduleConfig, "Weather", "Options",
			QT_TRANSLATE_NOOP("@default","Keep forecast in memory for:"), "KeepFor", 0, 24, 1, 1);
	ConfigDialog::addCheckBox(moduleConfig, "Weather", "Options",
			QT_TRANSLATE_NOOP("@default","Auto-download new forecast for me"),
			"bAuto", false);
	
	QStringList comboText = QStringList::split(',', tr("Today,Tomorrow,The day after tomorrow"));
	QStringList comboValues = QStringList::split(',', "0,1,2");
	
	ConfigDialog::addHBox("Weather", "Options", "hint_box");
	ConfigDialog::addCheckBox(moduleConfig, "Weather", "hint_box",
			QT_TRANSLATE_NOOP("@default","Show hint:"),
			"bHint", true);
	ConfigDialog::addLineEdit(moduleConfig, "Weather", "hint_box", "","HintText","",
			QT_TRANSLATE_NOOP("@default","%l - location\n%d - day\n%t - temperature\n%o - description\n%w - wind speed\n%p - pressure"), "hint_text");
	ConfigDialog::addComboBox(moduleConfig, "Weather", "hint_box", "", "HintDay", comboText, comboValues, "", "", "hint_day");
			
	ConfigDialog::addHBox("Weather", "Options", "desc_box");
	ConfigDialog::addCheckBox(moduleConfig, "Weather", "desc_box",
			QT_TRANSLATE_NOOP("@default","Update description text:"),
			"bDescription", false);
	ConfigDialog::addLineEdit(moduleConfig, "Weather", "desc_box","","DescriptionText","",
			QT_TRANSLATE_NOOP("@default","%l - location\n%d - day\n%t - temperature\n%o - description\n%w - wind speed\n%p - pressure"), "desc_text");
	ConfigDialog::addComboBox(moduleConfig, "Weather", "desc_box", "", "DescriptionDay", comboText, comboValues, "", "", "desc_day");
			
	ConfigDialog::addVGroupBox("Weather", "box", QT_TRANSLATE_NOOP("@default", "Hints options"));
	ConfigDialog::addSelectFont(moduleConfig, "Weather", "Hints options",
			QT_TRANSLATE_NOOP("@default", "Font:"), "HintFont");
	ConfigDialog::addColorButton(moduleConfig, "Weather", "Hints options",
			QT_TRANSLATE_NOOP("@default", "Font color:"), "FontColor", QColor(0,0,0));
	ConfigDialog::addColorButton(moduleConfig, "Weather", "Hints options",
			QT_TRANSLATE_NOOP("@default", "Background color:"), "BackgroundColor", QColor(255,255,255));
	ConfigDialog::addSpinBox(moduleConfig, "Weather", "Hints options",
			QT_TRANSLATE_NOOP("@default","Hint timeout:"), "HintTimeout", 0, 60, 1, 15);
	
	ConfigDialog::registerSlotOnCreate(this, SLOT(CreateConfigDialog()));
	ConfigDialog::registerSlotOnApply(this, SLOT(ApplyConfigDialog()));
	ConfigDialog::connectSlot("Weather", "Edit", SIGNAL(clicked()), this, SLOT(EditLocation()));
	ConfigDialog::connectSlot("Weather", "loc_list",
			SIGNAL(doubleClicked(QListViewItem*, const QPoint&, int)), this,
			SLOT(EditLocation()));
	ConfigDialog::connectSlot("Weather", "Clear", SIGNAL(clicked()), this, SLOT(ClearLocation()));
	ConfigDialog::connectSlot("Weather", "Find", SIGNAL(clicked()), this, SLOT(AutoFindLocation()));
	ConfigDialog::connectSlot("Weather", "Find all...", SIGNAL(clicked()), this, SLOT(AutoFindAllLocation()));
	ConfigDialog::connectSlot("Weather", "Keep forecast in memory for:", SIGNAL(valueChanged(int)),
			this, SLOT(changedKeepFor(int)));
	ConfigDialog::connectSlot("Weather", "Auto-download new forecast for me", SIGNAL(toggled(bool)),
			this, SLOT(checkedAutoLoad(bool)));
	ConfigDialog::connectSlot("Weather", "Show hint:", SIGNAL(toggled(bool)),
			this, SLOT(checkedShowHints(bool)));
	ConfigDialog::connectSlot("Weather", "Update description text:", SIGNAL(toggled(bool)),
			this, SLOT(checkedDescription(bool)));
	
	UserBox::userboxmenu->addItemAtPos(2,"ShowWeather", tr("Show weather"), this, SLOT(ShowWeather()));
	menuMyWeather = kadu->mainMenu()->insertItem(icons_manager.loadIcon("ShowWeather"),
			tr("Show weather"),this, SLOT(ShowMyWeather()), 0, -1, 0);
	
	if(moduleConfig->readBoolEntry("Weather", "ForecastFor", false))
		menuWeatherFor = kadu->mainMenu()->insertItem(icons_manager.loadIcon("ShowWeather"),
			tr("Show weather for..."), this, SLOT(ShowWeatherFor()), 0, -1, 1);
	else
		menuWeatherFor = -1;
	
	autoDownloadTimer = new QTimer(this);

	connect(UserBox::userboxmenu,SIGNAL(popup()),this,SLOT(PopupMenuCreate()));
	connect(autoDownloadTimer, SIGNAL(timeout()), this, SLOT(autoDownload()));
	connect(downloader, SIGNAL(finished()), this, SLOT(autoDownloadingFinished()));
	
	int keepfor = moduleConfig->readNumEntry("Weather", "KeepFor");
	if(keepfor > 0 && moduleConfig->readBoolEntry("Weather", "bAuto"))
	{
		autoDownloadTimer->start(keepfor*60*60*1000, false);
		autoDownload();
	}
}

Weather::~Weather()
{
	disconnect(downloader, SIGNAL(finished()), this, SLOT(autoDownloadingFinished()));
	disconnect(autoDownloadTimer, SIGNAL(timeout()), this, SLOT(autoDownload()));
	disconnect(UserBox::userboxmenu,SIGNAL(popup()),this,SLOT(PopupMenuCreate()));
		
	kadu->mainMenu()->removeItem(menuWeatherFor);
	kadu->mainMenu()->removeItem(menuMyWeather);
	UserBox::userboxmenu->removeItem(UserBox::userboxmenu->getItem(tr("Show weather")));

	ConfigDialog::disconnectSlot("Weather", "Update description text:",
			SIGNAL(toggled(bool)), this, SLOT(checkedDescription(bool)));
	ConfigDialog::disconnectSlot("Weather", "Show hint:",
			SIGNAL(toggled(bool)), this, SLOT(checkedShowHints(bool)));
	ConfigDialog::disconnectSlot("Weather", "Auto-download new forecast for me",
			SIGNAL(toggled(bool)), this, SLOT(checkedAutoLoad(bool)));
	ConfigDialog::disconnectSlot("Weather", "Keep forecast in memory for:",
			SIGNAL(valueChanged(int)), this, SLOT(changedKeepFor(int)));
	ConfigDialog::disconnectSlot("Weather", "loc_list",
			SIGNAL(doubleClicked(QListViewItem*, const QPoint&, int)), this, SLOT(EditLocation()));
	ConfigDialog::disconnectSlot("Weather", "Find all...", SIGNAL(clicked()), this, SLOT(AutoFindAllLocation()));
	ConfigDialog::disconnectSlot("Weather", "Find", SIGNAL(clicked()), this, SLOT(AutoFindLocation()));
	ConfigDialog::disconnectSlot("Weather", "Clear", SIGNAL(clicked()), this, SLOT(ClearLocation()));
	ConfigDialog::disconnectSlot("Weather", "Edit", SIGNAL(clicked()), this, SLOT(EditLocation()));
	ConfigDialog::unregisterSlotOnApply(this, SLOT(ApplyConfigDialog()));
	ConfigDialog::unregisterSlotOnCreate(this, SLOT(CreateConfigDialog()));
	
	ConfigDialog::removeControl("Weather", "Hint timeout:");
	ConfigDialog::removeControl("Weather", "Background color:");
	ConfigDialog::removeControl("Weather", "Font color:");
	ConfigDialog::removeControl("Weather", "Font:");
	ConfigDialog::removeControl("Weather", "Hints options");
	ConfigDialog::removeControl("Weather", "", "desc_day");
	ConfigDialog::removeControl("Weather", "", "desc_text");
	ConfigDialog::removeControl("Weather", "Update description text:");
	ConfigDialog::removeControl("Weather", "desc_box");
	ConfigDialog::removeControl("Weather", "", "hint_day");
	ConfigDialog::removeControl("Weather", "", "hint_text");
	ConfigDialog::removeControl("Weather", "Show hint:");
	ConfigDialog::removeControl("Weather", "hint_box");
	ConfigDialog::removeControl("Weather", "Auto-download new forecast for me");
	ConfigDialog::removeControl("Weather", "Keep forecast in memory for:");
	ConfigDialog::removeControl("Weather", "Show \"Forecast for...\" in main menu");
	ConfigDialog::removeControl("Weather", "Options");
	ConfigDialog::removeControl("Weather", "Find all...");
	ConfigDialog::removeControl("Weather", "Clear");
	ConfigDialog::removeControl("Weather", "Edit");
	ConfigDialog::removeControl("Weather", "Find");
	ConfigDialog::removeControl("Weather", "button_box");
	ConfigDialog::removeControl("Weather", "loc_list");
	ConfigDialog::removeControl("Weather", "Weather for contacts");
	ConfigDialog::removeControl("Weather", "box");
	
	ConfigDialog::removeTab("Weather");
	
	delete downloader;
	
	for(int i=0; i<10; i++)
		moduleConfig->writeEntry("LastLoc", QString("Location%1").arg(i+1), lastLocList[i]);
	
	moduleConfig->sync();
	delete moduleConfig;
}

void Weather::CreateConfigDialog()
{
	QListView* lv_loc=ConfigDialog::getListView("Weather","loc_list");
	lv_loc->setSorting(0);
	lv_loc->addColumn(tr("Contact"));
	lv_loc->addColumn(tr("Location code"));
	lv_loc->addColumn(tr("Weather server"));
	lv_loc->addColumn(tr("Config file"));
	lv_loc->setAllColumnsShowFocus(true);
	lv_loc->setColumnWidthMode(0, QListView::Maximum);
	lv_loc->setColumnWidthMode(1, QListView::Maximum);
	lv_loc->setColumnWidthMode(2, QListView::Maximum);
	lv_loc->setColumnWidthMode(3, QListView::Maximum);
	lv_loc->setResizeMode(QListView::LastColumn);
	//lv_loc->setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Ignored));
	
	QPushButton* b_edit=ConfigDialog::getPushButton("Weather","Edit");
	b_edit->setDefault(true);
	
	QString myUIN = config_file.readEntry("General","UIN");
	QString configfile = moduleConfig->readEntry("Configs", myUIN);
	ConfigFile wConfig(dataPath("kadu/modules/data/weather/"+configfile));
	QString configname = wConfig.readEntry("Header","Name");
	
	myNick = config_file.readEntry("General","Nick");
	QString myLoc = moduleConfig->readEntry("Locations", myUIN);
	int myUINint = config_file.readNumEntry("General","UIN");
	QListViewItem* myItem = new QListViewItem(lv_loc, myNick, myLoc, configname, configfile);
	myItem->setPixmap(0, icons_manager.loadIcon("ShowWeather"));
	
	int uin;
	for(UserList::ConstIterator u = userlist.begin(); u != userlist.end(); u++)
	{
		uin = (*u).uin();
		if(uin != 0 && uin != myUINint)
		{
			QString configfile = moduleConfig->readEntry("Configs",QString::number(uin));
			ConfigFile wConfig(dataPath("kadu/modules/data/weather/"+configfile));
			QString configname = wConfig.readEntry("Header","Name");
			new QListViewItem(lv_loc, (*u).altNick(), moduleConfig->readEntry("Locations",QString::number(uin)),
				configname, configfile);
		}
	}
	
	ConfigDialog::getSpinBox("Weather", "Keep forecast in memory for:")->setSuffix(tr(" hours"));
	ConfigDialog::getSpinBox("Weather", "Hint timeout:")->setSuffix(" s");
	
	checkedShowHints(moduleConfig->readBoolEntry("Weather","bHint", false));
	checkedDescription(moduleConfig->readBoolEntry("Weather","bDescription", false));
	checkedAutoLoad(moduleConfig->readBoolEntry("Weather","bAuto", false));
	changedKeepFor(moduleConfig->readNumEntry("Weather","KeepFor", 0));
	
	lastKeepValue = moduleConfig->readNumEntry("Weather", "KeepFor");
	lastAuto = moduleConfig->readBoolEntry("Weather", "bAuto");
	lastHints = moduleConfig->readBoolEntry("Weather", "bHint");
	lastDesc = moduleConfig->readBoolEntry("Weather", "bDescription");
}

void Weather::ApplyConfigDialog()
{	
	QListView* lv_loc=ConfigDialog::getListView("Weather","loc_list");
	QListViewItem* item = lv_loc->firstChild();

	QString altnick, uin;
	while(item != 0)
	{
		altnick = item->text(0);
		
		if(altnick == myNick)
			uin = config_file.readEntry("General","UIN");
		else
			uin = QString::number(userlist.byAltNick(altnick).uin());
		
		moduleConfig->writeEntry("Locations", uin, item->text(1));
		moduleConfig->writeEntry("Configs", uin, item->text(3));
		item = item->nextSibling();
	}
	
	if(moduleConfig->readBoolEntry("Weather", "ForecastFor") && menuWeatherFor == -1)
		menuWeatherFor = kadu->mainMenu()->insertItem(icons_manager.loadIcon("ShowWeather"),
			tr("Show weather for..."), this, SLOT(ShowWeatherFor()), 0, -1, 1);
	else if(!moduleConfig->readBoolEntry("Weather", "ForecastFor") && menuWeatherFor != -1)
	{
		//MessageBox::msg(QString("menuWeatherForL %1").arg(menuWeatherFor));
		kadu->mainMenu()->removeItem(menuWeatherFor);
		menuWeatherFor = -1;
	}
	
	bool bAuto = moduleConfig->readBoolEntry("Weather", "bAuto");
	int keepValue = moduleConfig->readNumEntry("Weather", "KeepFor");
	
	if((!bAuto || keepValue == 0) && autoDownloadTimer->isActive())
		autoDownloadTimer->stop();
	else if( (keepValue != 0 && bAuto) &&
	        ((keepValue != lastKeepValue) ||
	         (bAuto  && !lastAuto) ||
	         (moduleConfig->readBoolEntry("Weather", "bHint") &&  !lastHints) ||
	         (moduleConfig->readBoolEntry("Weather", "bDescription") && !lastDesc) ))
	{
		autoDownloadTimer->start(keepValue*60*60*1000, false);
		autoDownload();
	}
	
	lastAuto = bAuto;
	lastKeepValue = keepValue;
	lastHints = moduleConfig->readBoolEntry("Weather", "bHint");
	lastDesc = moduleConfig->readBoolEntry("Weather", "bDescription");
}

void Weather::EditLocation()
{
	QListView* lv_loc=ConfigDialog::getListView("Weather","loc_list");
	QListViewItem* item = lv_loc->currentItem();
	
	if(item == 0)
		return;
	
	int actual = weatherConfigFiles.findIndex(item->text(3));
	
	WeatherCodeDialog* wcd = new WeatherCodeDialog(item, moduleConfig,
		weatherConfigFiles, weatherConfigNames, actual, lastLocList, lv_loc);
	wcd->show();
}

void Weather::ClearLocation()
{
	QListView* lv_loc=ConfigDialog::getListView("Weather","loc_list");
	QListViewItem* listItem = lv_loc->currentItem();
	
	if(listItem == 0)
		return;
	
	listItem->setText(1, "");
	listItem->setText(2, "");
	listItem->setText(3, "");
}

void Weather::AutoFindLocation()
{
	QListView* lv_loc=ConfigDialog::getListView("Weather","loc_list");
	QListViewItem* item = lv_loc->currentItem();
	
	if(item == 0)
		return;
	
	AutoFindID* afi = new AutoFindID(lv_loc, moduleConfig,
		weatherConfigFiles, weatherConfigNames, lv_loc, item);
	afi->show();
}

void Weather::AutoFindAllLocation()
{
	QListView* lv_loc=ConfigDialog::getListView("Weather","loc_list");
	AutoFindID* afi = new AutoFindID(lv_loc, moduleConfig,
		weatherConfigFiles, weatherConfigNames, lv_loc);
	afi->show();
}

void Weather::checkedAutoLoad(bool check)
{
	ConfigDialog::getHBox("Weather", "hint_box")->setEnabled(check);
	ConfigDialog::getHBox("Weather", "desc_box")->setEnabled(check);
	//ConfigDialog::getComboBox("Weather", "Show day:")->setEnabled(check);
}

void Weather::checkedShowHints(bool check)
{
	ConfigDialog::getLineEdit("Weather", "", "hint_text")->setEnabled(check);
	ConfigDialog::getComboBox("Weather", "", "hint_day")->setEnabled(check);
}

void Weather::checkedDescription(bool check)
{
	ConfigDialog::getLineEdit("Weather", "", "desc_text")->setEnabled(check);
	ConfigDialog::getComboBox("Weather", "", "desc_day")->setEnabled(check);
}

void Weather::changedKeepFor(int value)
{
	QCheckBox* cbAuto = ConfigDialog::getCheckBox("Weather", "Auto-download new forecast for me");

	if(value == 0)
	{
		cbAuto->setEnabled(false);
		if(cbAuto->isChecked())
			checkedAutoLoad(false);
	}
	else if(!cbAuto->isEnabled())
	{
		cbAuto->setEnabled(true);
		if(cbAuto->isChecked())
			checkedAutoLoad(true);
	}
	
	moduleConfig->readBoolEntry("Weather","bAuto", false);
}

void Weather::PopupMenuCreate()
{
	UserBox *activeUserBox=kadu->userbox()->getActiveUserBox();
	if (activeUserBox==NULL)//to si zdarza...
		return;
	
	const UserListElement& user = (*activeUserBox->getSelectedUsers().begin());
	
	bool active = !moduleConfig->readEntry("Locations",QString::number(user.uin())).isNull();

	if(active){
		active = !moduleConfig->readEntry("Configs",QString::number(user.uin())).isNull();
		
		//if(!active && !moduleConfig->readEntry("Weather","ServerConfigFile").isNull())
		//	active = true;
	}
	UserBox::userboxmenu->setItemEnabled(UserBox::userboxmenu->getItem(tr("Show weather")), active);
}

void Weather::ShowWeather()
{
	UserBox *activeUserBox=kadu->userbox()->getActiveUserBox();
	if (activeUserBox==NULL)//to si zdarza...
		return;
	
	UserList userList = activeUserBox->getSelectedUsers();
	
	QString configFile = moduleConfig->readEntry("Configs",QString::number((*userList.begin()).uin()));
	QString locID = moduleConfig->readEntry("Locations",QString::number((*userList.begin()).uin()));
	
	ShowForecastDialog* sfd = new ShowForecastDialog(savedForecast, configFile, locID, moduleConfig);
	sfd->setCaption((*userList.begin()).altNick() + " - " + tr("Forecast"));
	sfd->show();
}

void Weather::ShowMyWeather()
{
	QString uin = config_file.readEntry("General","UIN");
	QString locid = moduleConfig->readEntry("Locations",uin);
	QString configFile = moduleConfig->readEntry("Configs",uin);
	if(locid.isEmpty() || configFile.isEmpty())
		return;
	
	ShowForecastDialog* sfd = new ShowForecastDialog(savedForecast,configFile,locid, moduleConfig);
	sfd->setCaption(tr("Forecast"));
	sfd->show();
}

void Weather::ShowWeatherFor()
{
	ShowForecastFor* sff = new ShowForecastFor(moduleConfig, savedForecast, weatherConfigFiles, weatherConfigNames, lastLocList);
	sff->show();
}

void Weather::autoDownload()
{
	QString uin = config_file.readEntry("General","UIN");
	QString locid = moduleConfig->readEntry("Locations", uin);
	QString cfg = moduleConfig->readEntry("Configs", uin);
	if(locid.isEmpty() || cfg.isEmpty())
		return;
	
	downloader->downloadForecast(cfg, locid);
}

void Weather::autoDownloadingFinished()
{
	savedForecast.add(forecast);
	
	if(moduleConfig->readNumEntry("Weather", "KeepFor") > 0 &&
		moduleConfig->readBoolEntry("Weather", "bAuto"))
	{
		if(moduleConfig->readBoolEntry("Weather", "bHint"))
		{
			const ForecastDay& day = forecast.Days[moduleConfig->readNumEntry("Weather", "HintDay")];
			hint_manager->addHint(parse(day, moduleConfig->readEntry("Weather", "HintText")),
				QPixmap(day["Icon"]),
				moduleConfig->readFontEntry("Weather", "HintFont"),
				moduleConfig->readColorEntry("Weather", "FontColor"),
				moduleConfig->readColorEntry("Weather", "BackgroundColor"),
				moduleConfig->readNumEntry("Weather", "HintTimeout"));
		}
		
		if(moduleConfig->readBoolEntry("Weather", "bDescription") && !gadu->status().isOffline())
		{
			const ForecastDay& day = forecast.Days[moduleConfig->readNumEntry("Weather", "DescriptionDay")];
			QString newStatusText = parse(day, moduleConfig->readEntry("Weather", "DescriptionText"));
			if(newStatusText != gadu->status().description() || !gadu->status().hasDescription())
				gadu->status().setDescription(newStatusText);
		}
	}
}

QString Weather::parse(const ForecastDay& day, QString str)
{
	str.replace("%l", forecast.LocationName);
	str.replace("%d", day["Name"]);
	str.replace("%t", day["Temperature"]);
	str.replace("%o", day["Description"]);
	str.replace("%w", day["Wind speed"]);
	str.replace("%p", day["Pressure"]);
	str.replace("&deg;","");
	str.replace("&nbsp;"," ");
	return str;
}

/**
	ShowForecastFor
**/

int ShowForecastFor::config_pos = 0;

ShowForecastFor::ShowForecastFor(ConfigFile* cf, ForecastContener& sf,
		const QStringList& wCfFiles, const QStringList& wCfNames, QStringList& lastLoc, QWidget* parent)
:
	QDialog(parent, "ShowForecastFor"),
	idSearch(cf, wCfFiles),
	moduleConfig(cf),
	savedForecast(sf),
	weatherConfigNames(wCfNames),
	weatherConfigFiles(wCfFiles),
	lastLocList(lastLoc)
{
	setWFlags(WDestructiveClose);
	setCaption(tr("Find location"));
	
	QHBox* config_box = new QHBox(this);
	new QLabel(tr("Weather server: "), config_box);
	c_config = new QComboBox(false,config_box);
	c_config->insertStringList(weatherConfigNames);
	c_config->setCurrentItem(config_pos);
	
	QHBox* search_box = new QHBox(this);
	new QLabel(QString(tr("Name: ")), search_box);
	c_name = new QComboBox(search_box);
	c_name->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed));
	c_name->setEditable(true);
	c_name->setFocus();
	
	for(int i=0; i<10; i++)
	{
		if(!lastLocList[i].isEmpty())
			c_name->insertItem(lastLocList[i], i);
	}
	c_name->setCurrentText("");
	
	QHBox* status_box = new QHBox(this);
	l_status = new QLabel(status_box);
	l_status->setMinimumWidth(100);
	l_status->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed));
	
	b_search = new QPushButton(status_box);
	b_search->setText(tr("Search"));
	b_search->setMaximumWidth(200);
	b_search->setSizePolicy(QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed));
	b_search->setIconSet(icons_manager.loadIcon("LookupUserInfo"));
	
	b_cancel = new QPushButton(status_box);
	b_cancel->setText(tr("Cancel"));
	b_cancel->setMaximumWidth(200);
	b_cancel->setSizePolicy(QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed));
	b_cancel->setIconSet(icons_manager.loadIcon("CloseWindowButton"));
	
	QGridLayout *grid= new QGridLayout(this, 1,0, 5, 10);
	grid->addWidget(config_box, 0, 0);
	grid->addWidget(search_box, 1, 0);
	grid->addWidget(status_box, 2, 0);
	
	connect(&idSearch, SIGNAL(finished()), this, SLOT(downloadingFinished()));
	connect(&idSearch, SIGNAL(error()), this, SLOT(downloadingError()));
	connect(b_search, SIGNAL(clicked()), this, SLOT(searchButton()));
	connect(b_cancel, SIGNAL(clicked()), this, SLOT(close()));
	connect(c_name, SIGNAL(activated(int)), this, SLOT(searchButton()));
}

ShowForecastFor::~ShowForecastFor()
{
	disconnect(c_name, SIGNAL(activated(int)), this, SLOT(searchButton()));
	disconnect(b_cancel, SIGNAL(clicked()), this, SLOT(close()));
	disconnect(b_search, SIGNAL(clicked()), this, SLOT(searchButton()));
	disconnect(&idSearch, SIGNAL(error()), this, SLOT(downloadingError()));
	disconnect(&idSearch, SIGNAL(finished()), this, SLOT(downloadingFinished()));
}

void ShowForecastFor::searchButton(void)
{
	QString loc_name = c_name->currentText();
	
	if(loc_name.isEmpty())
		return;

	config_pos = c_config->currentItem();
	l_status->setText(tr("Searching..."));
	idSearch.findID(loc_name, config_pos);
}

void ShowForecastFor::downloadingFinished(void)
{
	l_status->setText("");
	
	//const QStringList& names = idSearch.getNames();
	const QStringList& codes = idSearch.getCodes();
	
	if(codes.isEmpty())
	{
		MessageBox::msg(tr("Search location not found"));
		return;
	}
	
	QString location = c_name->currentText();
	if(lastLocList.find(location) == lastLocList.end())
	{
		lastLocList.push_front(location);
		c_name->insertItem(location, 0);
		if(lastLocList.count() > 10)
			lastLocList.remove(lastLocList.at(10));
	}

	ShowForecastDialog* sfd = new ShowForecastDialog(savedForecast,
		weatherConfigFiles[config_pos], *codes.begin(), moduleConfig);
	sfd->setCaption(tr("Forecast"));
	close();
	sfd->show();
}

void ShowForecastFor::downloadingError(void)
{
	l_status->setText("");
	MessageBox::wrn(tr("Downloading failed"));
}


/**
	AutoFindID
**/

AutoFindID::AutoFindID(QListView* list, ConfigFile* cf, const QStringList& wCfFiles,
				const QStringList& wCfNames, QWidget* parent, QListViewItem* item)
:
	QDialog(parent, "AutoFindID"),
	idSearch(cf, wCfFiles),
	lv_loc(list),
	listItem(0),
	weatherConfigFiles(wCfFiles),
	weatherConfigNames(wCfNames)
{
	setWFlags(WDestructiveClose);
	
	l_message = new QLabel(this);
	l_message->setText(tr("Press <b>Search</b> to start"));
	l_message->setAlignment(Qt::AlignTop);
	l_message->setMinimumWidth(300);
	l_message->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
	
	p_linking = new QProgressBar(this);
	
	QHBox* okcancel = new QHBox(this);
	
	b_search = new QPushButton(okcancel);
	b_search->setText(tr("Search"));
	b_search->setMaximumWidth(200);
	b_search->setSizePolicy(QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed));
	b_search->setIconSet(icons_manager.loadIcon("LookupUserInfo"));
	
	b_cancel = new QPushButton(okcancel);
	b_cancel->setText(tr("Close"));
	b_cancel->setMaximumWidth(200);
	b_cancel->setSizePolicy(QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed));
	b_cancel->setIconSet(icons_manager.loadIcon("CloseWindowButton"));
	
	QGridLayout *grid= new QGridLayout(this, 1, 0, 5, 10);
	grid->addWidget(l_message, 0, 0);
	grid->addWidget(p_linking, 1, 0);
	grid->addWidget(okcancel, 3, 0, Qt::AlignCenter);

	connect(gadu, SIGNAL(newSearchResults(SearchResults &, int, int)), this,
		SLOT(newSearchResults(SearchResults &, int, int)));
	connect(&idSearch, SIGNAL(finished()), this, SLOT(downloadingFinished()));
	connect(&idSearch, SIGNAL(error()), this, SLOT(downloadingError()));
	connect(b_search, SIGNAL(clicked()), this, SLOT(searchButton()));
	connect(b_cancel, SIGNAL(clicked()), this, SLOT(close()));
	
	if(item != 0)
	{
		setCaption(tr("Find location"));
		singleItem = true;
		b_search->hide();
		p_linking->setTotalSteps(2);
		p_linking->setProgress(0);
		nextItem(item);
	}
	else
	{
		setCaption(tr("Find locations"));
		singleItem = false;
		b_cancel->hide();
		p_linking->setTotalSteps(list->childCount()*2);
	}
}

AutoFindID::~AutoFindID()
{
	disconnect(b_cancel, SIGNAL(clicked()), this, SLOT(close()));
	disconnect(b_search, SIGNAL(clicked()), this, SLOT(searchButton()));
	disconnect(&idSearch, SIGNAL(error()), this, SLOT(downloadingError()));
	disconnect(&idSearch, SIGNAL(finished()), this, SLOT(downloadingFinished()));
	disconnect(gadu, SIGNAL(newSearchResults(SearchResults &, int, int)), this,
		SLOT(newSearchResults(SearchResults &, int, int)));
}

void AutoFindID::nextItem(QListViewItem* item)
{
	if(item == 0)
	{
		close();
		return;
	}
	else
		listItem = item;
	
	QString uin;
	altnick = item->text(0);
	if(altnick == config_file.readEntry("General","Nick"))
		uin = config_file.readEntry("General","UIN");
	else
		uin = QString::number(userlist.byAltNick(altnick).uin());

	l_message->setText("<b>" + altnick + ":</b><br>" + tr("Searching in public directory..."));
	
	searchRecord.clearData();
	searchRecord.reqUin(uin);
	gadu->searchInPubdir(searchRecord);
}

void AutoFindID::searchButton(void)
{
	b_search->hide();
	b_cancel->show();
	
	if(listItem != 0)
		nextItem(listItem);
	else
	{
		p_linking->setProgress(0);
		nextItem(lv_loc->firstChild());
	}
}

void AutoFindID::newSearchResults(SearchResults& searchResults, int seq, int fromUin)
{
	city = searchResults.first().City;
	if(city.isEmpty())
	{
		city = searchResults.first().FamilyCity;
		if(city.isEmpty())
		{
			p_linking->setProgress(p_linking->progress()+2);
			nextItem(singleItem ? 0 : listItem->nextSibling());
			return;
		}
	}
	
	indexOfConfigFile = 0;
	
	l_message->setText("<b>" + altnick + ":</b><br>" + tr("Searching for ") +
			city + tr(" in ") + weatherConfigNames[indexOfConfigFile] + "...");
			
	p_linking->setProgress(p_linking->progress()+1);
	idSearch.findID(city, indexOfConfigFile);
}

void AutoFindID::downloadingFinished(void)
{
	const QStringList& codes = idSearch.getCodes();
	
	if(codes.isEmpty())
	{
		++indexOfConfigFile;
		if(indexOfConfigFile < (int)weatherConfigNames.count())
		{
			l_message->setText("<b>" + altnick + ":</b><br>" + tr("Searching for ") +
			city + tr(" in ") + weatherConfigNames[indexOfConfigFile] + "...");
			idSearch.findID(city, indexOfConfigFile);
			return;
		}
	}
	else
	{
		listItem->setText(1, *codes.begin());
		listItem->setText(2, weatherConfigNames[indexOfConfigFile]);
		listItem->setText(3, weatherConfigFiles[indexOfConfigFile]);
	}
	p_linking->setProgress(p_linking->progress()+1);
	nextItem(singleItem ? 0 : listItem->nextSibling());
}

void AutoFindID::downloadingError(void)
{
	l_message->setText(tr("<b>Communication error</b><br>Connection timeout"));
	p_linking->setProgress(p_linking->progress()-1);
	b_search->setText(tr("Resume"));
	b_search->show();
	b_cancel->show();
}
