/***************************************************************************
 *   Weather module for Kadu communicator                                  *
 *   2004 Micha Maek (bysk)                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef WEATHERPARSER_H
#define WEATHERPARSER_H

#include <qstring.h>
#include <qcstring.h>
#include <qstringlist.h>
#include <qdatetime.h>

#include "config_file.h"


/**
	Zawiera list wartoci opisujacych pogod. Dozwolone wartoci:
	"Name", "Icon", "Temperature", "Pressure", "Rain", "Snow", "Wind Speed", "Description"
	"Humidity", "Dew point", "Visibility"
**/
typedef QMap<QString, QString> ForecastDay;

/**
	Zawiera informacj o jednym dniu z prognozy pogody.
	LocationName - nazwa miejscowoci
	LocationID - kod miejscowoci
	config - nazwa pliku konfiguracyjnego
	loadTime - data pobrania prognozy
	Days - poszczeglne dni prognozy
**/
struct Forecast
{
	QString LocationName;
	QString LocationID;
	QValueList<ForecastDay> Days;
	QString config;
	QString serverName;
	QTime loadTime;
};

/**
	Zajmuje si wyowieniem informacji pogodowych ze strony WWW.
**/
class WeatherParser
{
	private:
	
		struct WDataValue
		{
			QString Name;
			QString Start;
			QString End;
			QString Content;
		};
		
		const QString& strPage;
		ConfigFile* config;
		
		void getDataValue(WDataValue& wdata, int& cursor, ConfigFile* wConfig, bool CaseSensitive);
		QString tagClean(QString str);
		
	public:
		WeatherParser(QString& page, ConfigFile* cf);
		
		void getData(ConfigFile* wConfig, Forecast& forecast, const QString& iconPath);
		void getSearch(QStringList& names, QStringList& codes, ConfigFile* wConfig);
		QString getFastSearch(const QString& link, const ConfigFile& wConfig);
};

/**
	Ma za zadanie przechowywa cignite prognozy.
	Prognozy identyfikowane s poprzez kod miejscowoci oraz
	serwer (nazwa pliku konfiguracyjnego). Oznacza to, e w kontenerze
	nie ma dwch prognoz dla tej samej miejscowoci z tym samym serwerem.
	
	Funkcja deleteObsolete() usuwa z pojemnika przestarzae prognozy
**/
class ForecastContener
{
	QValueList<Forecast> list;

public:
	bool getForecast(const QString& configFile, const QString& location, Forecast& forecast, int hours);
	void deleteObsolete(int hours);
	void add(const Forecast& forecast);
};

#endif
